<?php

namespace Emp\Employee\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Emp\Employee\Http\Requests\EmployeeRequest;
use Emp\Employee\Interfaces\EmployeeRepositoryInterface;
use Emp\Employee\Models\Employee;
use Emp\Employee\Forms\Employee as Form;

/**
 * APIController  class for employee.
 */
class EmployeeAPIController extends BaseController
{

    /**
     * Initialize employee resource controller.
     *
     * @param type EmployeeRepositoryInterface $employee
     *
     * @return null
     */
    public function __construct(EmployeeRepositoryInterface $employee)
    {
        parent::__construct();
        $this->repository = $employee;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Emp\Employee\Repositories\Criteria\EmployeeResourceCriteria::class);
    }

    /**
     * Display a list of employee.
     *
     * @return Response
     */
    public function index(EmployeeRequest $request)
    {
        return $this->repository
            ->setPresenter(\Emp\Employee\Repositories\Presenter\EmployeePresenter::class)
            ->paginate();
    }

    /**
     * Display employee.
     *
     * @param Request $request
     * @param Model   $employee
     *
     * @return Response
     */
    public function show(EmployeeRequest $request, Employee $employee)
    {
        return $employee->setPresenter(\Emp\Employee\Repositories\Presenter\EmployeeListPresenter::class);
        ;
    }

    /**
     * Create new employee.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(EmployeeRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('employee::employee.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('employee/employee/' . $employee->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('employee/employee');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the employee.
     *
     * @param Request $request
     * @param Model   $employee
     *
     * @return Response
     */
    public function update(EmployeeRequest $request, Employee $employee)
    {
        try {
            $data = $request->all();

            $employee->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('employee::employee.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('employee/employee/' . $employee->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('employee/employee/' . $employee->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the employee.
     *
     * @param Model   $employee
     *
     * @return Response
     */
    public function destroy(EmployeeRequest $request, Employee $employee)
    {
        try {
            $employee->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('employee::employee.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('employee/employee/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('employee/employee/' . $employee->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
